package com.shabinder.common.di.providers

import co.touchlab.kermit.Kermit
import com.shabinder.common.di.Dir
import com.shabinder.common.di.finalOutputDir
import com.shabinder.common.di.providers.requests.audioToMp3.AudioToMp3
import com.shabinder.common.di.providers.requests.saavn.JioSaavnRequests
import com.shabinder.common.di.utils.removeIllegalChars
import com.shabinder.common.models.DownloadStatus
import com.shabinder.common.models.PlatformQueryResult
import com.shabinder.common.models.SpotiFlyerException
import com.shabinder.common.models.TrackDetails
import com.shabinder.common.models.event.coroutines.SuspendableEvent
import com.shabinder.common.models.saavn.SaavnSong
import com.shabinder.common.models.spotify.Source
import io.ktor.client.*

class SaavnProvider(
    override val httpClient: HttpClient,
    override val logger: Kermit,
    override val audioToMp3: AudioToMp3,
    private val dir: Dir,
) : JioSaavnRequests {

    suspend fun query(fullLink: String): SuspendableEvent<PlatformQueryResult, Throwable> = SuspendableEvent {
        PlatformQueryResult(
            folderType = "",
            subFolder = "",
            title = "",
            coverUrl = "",
            trackList = listOf(),
            Source.JioSaavn
        ).apply {
            val pageLink = fullLink.substringAfter("saavn.com/").substringBefore("?")
            when {
                pageLink.contains("/song/",true) -> {
                    getSong(fullLink).value.let {
                        folderType = "Tracks"
                        subFolder = ""
                        trackList = listOf(it).toTrackDetails(folderType, subFolder)
                        title = it.song
                        coverUrl = it.image.replace("http:", "https:")
                    }
                }
                pageLink.contains("/album/",true) -> {
                    getAlbum(fullLink).value.let {
                        folderType = "Albums"
                        subFolder = removeIllegalChars(it.title)
                        trackList = it.songs.toTrackDetails(folderType, subFolder)
                        title = it.title
                        coverUrl = it.image.replace("http:", "https:")
                    }
                }
                pageLink.contains("/featured/",true) -> { // Playlist
                    getPlaylist(fullLink).value.let {
                        folderType = "Playlists"
                        subFolder = removeIllegalChars(it.listname)
                        trackList = it.songs.toTrackDetails(folderType, subFolder)
                        coverUrl = it.image.replace("http:", "https:")
                        title = it.listname
                    }
                }
                else -> {
                    throw SpotiFlyerException.LinkInvalid(fullLink)
                }
            }
        }
    }

    private fun List<SaavnSong>.toTrackDetails(type: String, subFolder: String): List<TrackDetails> = this.map {
        TrackDetails(
            title = it.song,
            artists = it.artistMap.keys.toMutableSet().apply { addAll(it.singers.split(",")) }.toList(),
            durationSec = it.duration.toInt(),
            albumName = it.album,
            albumArtPath = dir.imageCacheDir() + (it.image.substringBeforeLast('/').substringAfterLast('/')) + ".jpeg",
            year = it.year,
            comment = it.copyright_text,
            trackUrl = it.perma_url,
            videoID = it.id,
            downloadLink = it.media_url, // Downloadable Link
            downloaded = it.updateStatusIfPresent(type, subFolder),
            albumArtURL = it.image.replace("http:", "https:"),
            lyrics = it.lyrics ?: it.lyrics_snippet,
            source = Source.JioSaavn,
            outputFilePath = dir.finalOutputDir(it.song, type, subFolder, dir.defaultDir(), /*".m4a"*/)
        )
    }

    private fun SaavnSong.updateStatusIfPresent(folderType: String, subFolder: String): DownloadStatus {
        return if (dir.isPresent(
                dir.finalOutputDir(
                        song,
                        folderType,
                        subFolder,
                        dir.defaultDir()
                    )
            )
        ) { // Download Already Present!!
            DownloadStatus.Downloaded.also {
                downloaded = it
            }
        } else downloaded
    }
}
